const std = @import("std");
const stdout = std.io.getStdOut().writer();
const stderr = std.io.getStdErr().writer();

fn print(comptime str: []const u8, params: anytype) void {
    stdout.print(str ++ "\n", params) catch {};
}

fn pErr(comptime str: []const u8, params: anytype) void {
    stderr.print(str ++ "\n", params) catch {};
}

fn exit(comptime str: []const u8, params: anytype, exitCode: u8) void {
    pErr(str, params);
    std.os.exit(exitCode);
}

fn usage() void {
    pErr("reader <path>", .{});
    exit("", .{}, 1);
}

const LOCAL_FILE_HEADER_SIGNATURE = 0x04034b50;
const ZipFileHeader = struct {
    version: u16,
    general: u16,
    compression_method: u16,
    last_mod_time: u16,
    last_mod_date: u16,
    crc_32: u32,
    compressed_size: u32,
    uncompressed_size: u32,
    file_name_length: u16,
    extra_field_length: u16,

    file_name: []u8,
    extra_field: []u8,

    allocator: std.mem.Allocator,

    const Self = @This();

    fn init(allocator: std.mem.Allocator, reader: std.fs.File.Reader) !Self {
        if (try reader.readInt(u32, .Big) == LOCAL_FILE_HEADER_SIGNATURE) {
            return error.InvalidError;
        }

        var self = Self{
            .allocator = allocator,
            .version = try reader.readInt(u16, .Big),
            .general = try reader.readInt(u16, .Big),
            .compression_method = try reader.readInt(u16, .Big),
            .last_mod_time = try reader.readInt(u16, .Big),
            .last_mod_date = try reader.readInt(u16, .Big),
            .crc_32 = try reader.readInt(u32, .Big),
            .compressed_size = try reader.readInt(u32, .Big),
            .uncompressed_size = try reader.readInt(u32, .Big),
            .file_name_length = try reader.readInt(u16, .Big),
            .extra_field_length = try reader.readInt(u16, .Big),
            .file_name = undefined,
            .extra_field = undefined,
        };

        self.file_name = try allocator.alloc(u8, self.file_name_length);
        self.extra_field = try allocator.alloc(u8, self.extra_field_length);

        _ = try reader.read(self.file_name);
        _ = try reader.read(self.extra_field);

        return self;
    }

    fn deinit(self: *Self) void {
        self.allocator.free(self.file_name);
        self.allocator.free(self.extra_field);
    }
};

pub fn main() !void {
    var args = std.process.args();

    var filePath: ?[]const u8 = null;

    // Skip the current path
    _ = args.next();
    while (args.next()) |arg| {
        if (filePath == null) {
            filePath = arg;
        } else {
            pErr("Invalid argument: {s}", .{arg});
            usage();
        }
    }

    if (filePath == null) {
        pErr("File path not provided. Please provide a path", .{});
        usage();
    }

    var gpa = std.heap.GeneralPurposeAllocator(.{}){};
    defer _ = gpa.deinit();

    var allocator = gpa.allocator();

    var book_path = try std.fs.realpathAlloc(allocator, filePath.?);
    defer allocator.free(book_path);

    var file = std.fs.openFileAbsolute(book_path, .{}) catch |err| {
        exit("Please provide a file path! Error: {?}", .{err}, 1);
        return err;
    };
    defer file.close();

    var stat = try file.stat();

    if (stat.kind != .File) {
        exit("Please provide a valid file", .{}, 1);
    }

    var reader = file.reader();

    var first_file = try ZipFileHeader.init(allocator, reader);
    defer first_file.deinit();

    print("G: {}", .{first_file.file_name_length});
    print("T: {s}", .{first_file.file_name});
    print("O: {s}", .{first_file.extra_field});
}
